/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "TrashFunctions.h"

extern LPSHELLFOLDER2	m_pRecycleBin;

int GetTrashCount(bool getHardDisks)
{
	DWORD			dwSize		= GetLogicalDriveStrings(0, NULL);
    LPTSTR			pszDrives	= (LPTSTR)malloc((dwSize + 2) * sizeof (TCHAR));
	HRESULT			hr			= S_OK;
	int				iResult		= 0;
	
	// If we were able to get the names of available drives, then we'll check if
	// objects are available in their local RBin. If so, we'll add their name to
	// the button's menu as an option for the user to empty them individually.
	if (NULL != pszDrives)
	{
		LPTSTR pstr = pszDrives;
		SHQUERYRBINFO qrbi;

		GetLogicalDriveStrings((dwSize + 2) * sizeof (TCHAR),(LPTSTR)pszDrives);
		
		while (TCHAR ('\0') != *pstr)
		{
			if ( getHardDisks )
			{
				 // Get the drive type.
				int nDriveType = GetDriveType (pstr);

				switch ( nDriveType )
				{
					case DRIVE_FIXED :
						iResult++;

					break;

					default:
						break;
				}
			}
			else
			{
				ZeroMemory (&qrbi, sizeof (qrbi));
				qrbi.cbSize = sizeof (qrbi);
				hr = SHQueryRecycleBin (pstr, &qrbi);
				if (SUCCEEDED (hr))
				{
					if (0 != qrbi.i64NumItems)
					{
						iResult += qrbi.i64NumItems;
					}
				}
			}
			pstr += _tcslen(pstr) + sizeof (TCHAR);
		}
		free (pszDrives);
	}

	return iResult;
}


string GetTrashSize()
{
	LPMALLOC		pMalloc = NULL;
	TCHAR			szTemp[MAX_PATH];
	LPENUMIDLIST	penumFiles;
	LPITEMIDLIST	pidl = NULL;
	SHELLDETAILS	sd;
	int				iItem = 0;
	int				iSubItem = 0;
	int				iIndex = -1;
	SFGAOF			sg = SFGAO_VALIDATE;
	HRESULT			hr = S_OK;

	int size = 0;
	CString strSize = "";
	
	SHGetMalloc(&pMalloc); // windows memory management pointer needed later
	
	// Get the list of available objects
	hr = m_pRecycleBin->EnumObjects(/*m_hWnd*/NULL, SHCONTF_FOLDERS | SHCONTF_NONFOLDERS| SHCONTF_INCLUDEHIDDEN, &penumFiles);
	if (SUCCEEDED (hr))
	{
		// Iterate through list 
		while (penumFiles->Next(1, &pidl, NULL) != S_FALSE)
		{
			hr = m_pRecycleBin->GetDetailsOf(pidl, 4, &sd);
			if (SUCCEEDED (hr))
			{
				switch (sd.str.uType)
				{
					case STRRET_CSTR:
						_tcscpy (szTemp, sd.str.cStr);
						break;
					case STRRET_OFFSET:
						break;
					case STRRET_WSTR:
						WideCharToMultiByte (CP_ACP, 0, sd.str.pOleStr, -1, szTemp, sizeof (szTemp), NULL, NULL);
						pMalloc->Free(sd.str.pOleStr);
						break;	
				}

				strSize = szTemp;
				strSize.Replace(" KB", "");
				strSize.Replace(".", "");

				size = size + atoi(strSize);
			}
			pMalloc->Free(pidl);
		}
	}

	if (NULL != penumFiles)
	{
		penumFiles->Release ();
		penumFiles = NULL;
	}

	pMalloc->Release();

	return formatByteSize(size);
}

bool GetRecycleFolder()
{
	bool			bReturn			= false;
	LPMALLOC		pMalloc			= NULL;
	LPSHELLFOLDER	pDesktop		= NULL;
	LPITEMIDLIST	pidlRecycleBin	= NULL;
	HRESULT			hr				= S_OK;
	
	SHGetMalloc(&pMalloc); // windows memory management pointer needed later
	if (NULL != m_pRecycleBin)
	{
		m_pRecycleBin->Release();
		m_pRecycleBin = NULL;
	}
	
	if ((SUCCEEDED (SHGetDesktopFolder(&pDesktop))) &&
		(SUCCEEDED (SHGetSpecialFolderLocation (NULL, CSIDL_BITBUCKET, &pidlRecycleBin))))
	{
		if (SUCCEEDED (pDesktop->BindToObject(pidlRecycleBin, NULL, IID_IShellFolder2, (LPVOID *)&m_pRecycleBin)))
		{
			bReturn = true;
		}
	}
	
	if (NULL != pidlRecycleBin)
		pMalloc->Free (pidlRecycleBin);
	if (NULL != pDesktop)
		pDesktop->Release();
	pMalloc->Release();
	
	return bReturn;
}

static const char *units[] = {"KB", "MB", "GB", "TB", "PB", "EB"};

string formatByteSize(int kbyteSize)
{
	char buffer[32] = {0};

	int i = 0;

	if(kbyteSize & 0xFFFC000000000000)
		i++;
	if(kbyteSize & 0xFFFFFF0000000000)
		i++;
	if(kbyteSize & 0xFFFFFFFFC0000000)
		i++;
	if(kbyteSize & 0xFFFFFFFFFFF00000)
		i++;
	if(kbyteSize & 0xFFFFFFFFFFFFFC00)
		i++;

	int divisor = 1 << (i * 10);
	int quotient = kbyteSize / divisor;
	int remainder = ((kbyteSize % divisor) * 100) / divisor;

	if(quotient > 9 && quotient < 100)
	{
		if ( remainder > 99 )
			remainder = remainder / 10;
	}
	else if(quotient >= 100)
	{
		remainder = 0;
	}

	if(remainder > 0)
		StringCchPrintf(buffer, 32, "%d.%02d", (int) quotient, (int) remainder);
	else
		StringCchPrintf(buffer, 32, "%d", (int) quotient);

	StringCchCat(buffer, 32, " ");
	StringCchCat(buffer, 32, units[i]);

	return string (buffer);
}